@@ -31,9 +31,9 @@ function UICasebook:UICasebook(ui, disease_selection)
     self.background = gfx:loadRaw("DrugN01V", 640, 480, "QData", "QData", "DrugN01V.pal", true)
     local palette = gfx:loadPalette("QData", "DrugN01V.pal", true)
     self.panel_sprites = gfx:loadSpriteTable("QData", "DrugN02V", true, palette)
-    self.title_font = gfx:loadFont("QData", "Font25V", false, palette)
-    self.selected_title_font = gfx:loadFont("QData", "Font26V", false, palette)
-    self.drug_font = gfx:loadFont("QData", "Font24V", false, palette)
+    self.title_font = gfx:loadFontAndSpriteTable("QData", "Font25V", false, palette)
+    self.selected_title_font = gfx:loadFontAndSpriteTable("QData", "Font26V", false, palette)
+    self.drug_font = gfx:loadFontAndSpriteTable("QData", "Font24V", false, palette)
   end) then
     ui:addWindow(UIInformation(ui, {_S.errors.dialog_missing_graphics}))
     self:close()
@@ -399,9 +399,9 @@ function UICasebook:afterLoad(old, new)
     self.background = gfx:loadRaw("DrugN01V", 640, 480, "QData", "QData", "DrugN01V.pal", true)
     local palette = gfx:loadPalette("QData", "DrugN01V.pal", true)
     self.panel_sprites = gfx:loadSpriteTable("QData", "DrugN02V", true, palette)
-    self.title_font = gfx:loadFont("QData", "Font25V", false, palette)
-    self.selected_title_font = gfx:loadFont("QData", "Font26V", false, palette)
-    self.drug_font = gfx:loadFont("QData", "Font24V", false, palette)
+    self.title_font = gfx:loadFontAndSpriteTable("QData", "Font25V", false, palette)
+    self.selected_title_font = gfx:loadFontAndSpriteTable("QData", "Font26V", false, palette)
+    self.drug_font = gfx:loadFontAndSpriteTable("QData", "Font24V", false, palette)
   end
 
   UIFullscreen.afterLoad(self, old, new)
